package com.qmth.distributed.print.api;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.gson.reflect.TypeToken;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.dto.report.*;
import com.qmth.distributed.print.business.bean.result.ScoreResult;
import com.qmth.distributed.print.business.bean.result.report.PaperStructResult;
import com.qmth.distributed.print.business.bean.result.report.ReportResult;
import com.qmth.distributed.print.business.entity.TCPaperStruct;
import com.qmth.distributed.print.business.entity.TRBasicInfo;
import com.qmth.distributed.print.business.entity.TRExamStudent;
import com.qmth.distributed.print.business.service.*;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicExam;
import com.qmth.teachcloud.common.entity.BasicSemester;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.log.CustomizedOperationTypeEnum;
import com.qmth.teachcloud.common.service.BasicSemesterService;
import com.qmth.teachcloud.common.util.GsonUtil;
import com.qmth.teachcloud.common.util.JacksonUtil;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import io.swagger.annotations.*;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * <p>
 * 报告基本情况表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Api(tags = "课程目标达成度-报告基本情况Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_COURSE_DEGREE)
public class TRBasicInfoController {

    @Resource
    PrintCommonService printCommonService;

    @Resource
    MarkPaperService markPaperService;

    @Resource
    TRBasicInfoService trBasicInfoService;

    @Resource
    BasicSemesterService basicSemesterService;

    @Resource
    BasicExamService basicExamService;

    @Resource
    TCPaperStructService tcPaperStructService;

    @Resource
    TRExamStudentService trExamStudentService;

    @ApiOperation(value = "报告管理列表")
    @RequestMapping(value = "/report/list", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = ScoreResult.class)})
    public Object reportList(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                             @ApiParam(value = "课程编码") @RequestParam(required = false) String courseCode,
                             @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                             @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(printCommonService.scoreList(new Page<>(pageNumber, pageSize), examId, courseCode));
    }

    @ApiOperation(value = "查看报告")
    @RequestMapping(value = "/report/view", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    @ApiResponses({@ApiResponse(code = 200, message = "查看报告", response = ReportResult.class)})
    @Transactional
    public Object reportView(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                             @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                             @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        Objects.requireNonNull(markPaper, "未找到科目信息");

        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        TRBasicInfo trBasicInfo = trBasicInfoService.getOne(new QueryWrapper<TRBasicInfo>().lambda().eq(TRBasicInfo::getExamId, examId).eq(TRBasicInfo::getCourseCode, courseCode).eq(TRBasicInfo::getPaperNumber, paperNumber));

        ReportCourseBasicInfoDto reportCourseBasicInfoDto = null;//课程基本情况
        ReportCourseEvaluationSpreadDto reportCourseEvaluationSpreadDto = null;//课程目标考核分布
        ReportCourseEvaluationResultDto reportCourseEvaluationResultDto = null;//课程考核成绩评价结果
        ReportCourseEvaluationResultDetailDto reportCourseEvaluationResultDetailDto = null;//课程目标达成评价明细结果
        if (Objects.isNull(trBasicInfo)) {
            BasicExam basicExam = basicExamService.getById(examId);
            Objects.requireNonNull(basicExam, "未找到考试信息");

            BasicSemester basicSemester = basicSemesterService.getById(basicExam.getSemesterId());
            Objects.requireNonNull(basicSemester, "未找到学期信息");

            //TODO 有测试数据，待肖飞补充授课对象和任课老师数据
            trBasicInfo = new TRBasicInfo(examId, courseCode, markPaper.getCourseName(), paperNumber, basicSemester.getName(), "测试班级1", "测试老师1", 40, sysUser.getId());

            //测试数据
            //课程基本情况
            reportCourseBasicInfoDto = new ReportCourseBasicInfoDto(trBasicInfo);

            //课程目标考核分布-题目信息
            TCPaperStruct tcPaperStruct = tcPaperStructService.getOne(new QueryWrapper<TCPaperStruct>().lambda().eq(TCPaperStruct::getExamId, examId).eq(TCPaperStruct::getCourseCode, courseCode).eq(TCPaperStruct::getPaperNumber, paperNumber));
            Objects.requireNonNull(tcPaperStruct, "未找到试卷蓝图信息");

            List<PaperStructResult> paperStructResultList = GsonUtil.fromJson(tcPaperStruct.getPaperStruct(), new TypeToken<List<PaperStructResult>>() {
            }.getType());

            reportCourseEvaluationSpreadDto = new ReportCourseEvaluationSpreadDto(paperStructResultList);

            //课程目标考核分布-分数图
            reportCourseEvaluationSpreadDto.setScoreList(trBasicInfoService.getScoreList(trBasicInfo, markPaper));
            trBasicInfo.setCourseEvaluationSpread(JacksonUtil.parseJson(reportCourseEvaluationSpreadDto));

            Map<String, Object> paramsMap = trBasicInfoService.getReportCalculateParams(trBasicInfo, paperStructResultList);
            paramsMap = trBasicInfoService.reportCalculateMaxMinAvgSumScore(paramsMap);
            reportCourseEvaluationResultDto = trBasicInfoService.reportCalculateCourseEvaluationResult(paramsMap);

            trBasicInfo.setCourseEvaluationResult(JacksonUtil.parseJson(reportCourseEvaluationResultDto));
            //课程目标达成评价明细结果-考生集合
            List<ReportExamStudentDto> examStudentNewList = trExamStudentService.getExamStudentReportDetail(trBasicInfo, paramsMap);
            reportCourseEvaluationResultDetailDto = new ReportCourseEvaluationResultDetailDto(examStudentNewList);
        } else {
            reportCourseBasicInfoDto = new ReportCourseBasicInfoDto(trBasicInfo);
            reportCourseEvaluationSpreadDto = JSONObject.parseObject(trBasicInfo.getCourseEvaluationSpread(), ReportCourseEvaluationSpreadDto.class);
            reportCourseEvaluationResultDto = JSONObject.parseObject(trBasicInfo.getCourseEvaluationResult(), ReportCourseEvaluationResultDto.class);

            List<TRExamStudent> trExamStudentList = trExamStudentService.list(new QueryWrapper<TRExamStudent>().lambda().eq(TRExamStudent::getrBasicInfoId, trBasicInfo.getId()));
            if (!CollectionUtils.isEmpty(trExamStudentList)) {
                List<ReportExamStudentDto> examStudentList = new ArrayList<>(trExamStudentList.size());
                for (TRExamStudent trExamStudent : trExamStudentList) {
                    examStudentList.add(new ReportExamStudentDto(trExamStudent));
                }
                reportCourseEvaluationResultDetailDto = new ReportCourseEvaluationResultDetailDto(examStudentList);
            }
        }

        trBasicInfo.updateInfo(sysUser.getId());
        //课程目标达成评价明细结果-课程目标达成评价值图
        trBasicInfoService.saveOrUpdate(trBasicInfo);

        ReportCommonDto reportCommonDto = new ReportCommonDto(examId, courseCode, markPaper.getCourseName(), paperNumber);//公共信息
        trBasicInfo.setReportResult(new ReportResult(reportCommonDto, reportCourseBasicInfoDto, reportCourseEvaluationSpreadDto, reportCourseEvaluationResultDto, reportCourseEvaluationResultDetailDto));
        return ResultUtil.ok(trBasicInfo.getReportResult());
    }

    @ApiOperation(value = "保存报告")
    @RequestMapping(value = "/report/save", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "保存报告", response = Object.class)})
    public Object reportSave(@ApiParam(value = "保存报告结构", required = true) @Valid @RequestBody TRBasicInfo trBasicInfo, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        TRBasicInfo trBasicInfoDb = trBasicInfoService.getOne(new QueryWrapper<TRBasicInfo>().lambda().eq(TRBasicInfo::getExamId, trBasicInfo.getExamId()).eq(TRBasicInfo::getCourseCode, trBasicInfo.getCourseCode()).eq(TRBasicInfo::getPaperNumber, trBasicInfo.getPaperNumber()));
        if (Objects.isNull(trBasicInfoDb)) {
            trBasicInfoDb = new TRBasicInfo(trBasicInfoDb, sysUser.getId());
        } else {
            trBasicInfoDb.updateInfo(trBasicInfo, sysUser.getId());
        }
        return ResultUtil.ok(trBasicInfoService.saveOrUpdate(trBasicInfo));
    }

    @ApiOperation(value = "导出报告")
    @RequestMapping(value = "/report/export", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.EXPORT)
    @ApiResponses({@ApiResponse(code = 200, message = "下载成功", response = Object.class)})
    public void reportExport(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                             @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                             @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) throws IOException {
        TRBasicInfo trBasicInfo = trBasicInfoService.getOne(new QueryWrapper<TRBasicInfo>().lambda().eq(TRBasicInfo::getExamId, examId).eq(TRBasicInfo::getCourseCode, courseCode).eq(TRBasicInfo::getPaperNumber, paperNumber));
        Objects.requireNonNull(trBasicInfo, "没有报告信息");
    }
}
